@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2020-2021 Fanout, Inc.
+ * Copyright (C) 2020-2022 Fanout, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,15 +14,49 @@
  * limitations under the License.
  */
 
+use crate::future::{AsyncWrite, AsyncWriteExt};
 use std::cell::RefCell;
 use std::cmp;
 use std::io;
 use std::io::{Read, Write};
+use std::mem::{self, MaybeUninit};
 use std::rc::Rc;
 
 pub const VECTORED_MAX: usize = 8;
 
+pub fn trim_for_display(s: &str, max: usize) -> String {
+    // NOTE: O(n)
+    let char_len = s.chars().count();
+
+    if char_len > max && max >= 7 {
+        let dist = max / 2;
+        let mut left_end = 0;
+        let mut right_start = 0;
+
+        // NOTE: O(n)
+        for (i, (pos, _)) in s.char_indices().enumerate() {
+            // dist guaranteed to be < char_len
+            if i == dist {
+                left_end = pos;
+            }
+
+            // (char_len - dist + 3) guaranteed to be < char_len
+            if i == char_len - dist + 3 {
+                right_start = pos;
+            }
+        }
+
+        let left = &s[..left_end];
+        let right = &s[right_start..];
+
+        format!("{}...{}", left, right)
+    } else {
+        s.to_owned()
+    }
+}
+
 pub trait RefRead {
+    fn len(&self) -> usize;
     fn get_ref(&self) -> &[u8];
     fn get_mut(&mut self) -> &mut [u8];
     fn consume(&mut self, amt: usize);
@@ -40,9 +74,9 @@ pub trait RefRead {
 
     fn get_mut_vectored<'data, 'bufs>(
         &'data mut self,
-        bufs: &'bufs mut [&'data mut [u8]],
+        bufs: &'bufs mut MaybeUninit<[&'data mut [u8]; VECTORED_MAX]>,
     ) -> &'bufs mut [&'data mut [u8]] {
-        assert!(bufs.len() >= 1);
+        let bufs = unsafe { bufs.assume_init_mut() };
 
         bufs[0] = self.get_mut();
 
@@ -51,6 +85,10 @@ pub trait RefRead {
 }
 
 impl RefRead for io::Cursor<&mut [u8]> {
+    fn len(&self) -> usize {
+        RefRead::get_ref(self).len()
+    }
+
     fn get_ref(&self) -> &[u8] {
         let pos = self.position() as usize;
 
@@ -114,6 +152,50 @@ pub fn write_vectored_offset(
     writer.write_vectored(&arr[..arr_len])
 }
 
+pub async fn write_vectored_offset_async(
+    writer: &mut dyn AsyncWrite,
+    bufs: &[&[u8]],
+    offset: usize,
+) -> Result<usize, io::Error> {
+    if bufs.is_empty() {
+        return Ok(0);
+    }
+
+    let mut offset = offset;
+    let mut start = 0;
+
+    while offset >= bufs[start].len() {
+        // on the last buf?
+        if start + 1 >= bufs.len() {
+            // exceeding the last buf is an error
+            if offset > bufs[start].len() {
+                return Err(io::Error::from(io::ErrorKind::InvalidInput));
+            }
+
+            return Ok(0);
+        }
+
+        offset -= bufs[start].len();
+        start += 1;
+    }
+
+    let mut arr = [io::IoSlice::new(&b""[..]); VECTORED_MAX];
+    let mut arr_len = 0;
+
+    for i in start..bufs.len() {
+        let buf = if i == start {
+            &bufs[i][offset..]
+        } else {
+            bufs[i]
+        };
+
+        arr[arr_len] = io::IoSlice::new(buf);
+        arr_len += 1;
+    }
+
+    writer.write_vectored(&arr[..arr_len]).await
+}
+
 pub trait LimitBufs<'a> {
     fn limit(&mut self, size: usize) -> &mut [&'a [u8]];
 }
@@ -137,6 +219,38 @@ impl<'a> LimitBufs<'a> for [&'a [u8]] {
     }
 }
 
+pub trait LimitBufsMut<'a> {
+    fn limit(&mut self, size: usize) -> &mut [&'a mut [u8]];
+}
+
+impl<'a> LimitBufsMut<'a> for [&'a mut [u8]] {
+    fn limit(&mut self, size: usize) -> &mut [&'a mut [u8]] {
+        let mut want = size;
+
+        for i in 0..self.len() {
+            let buf_len = self[i].len();
+
+            if buf_len >= want {
+                // SAFETY: shrinking the length of an existing slice requires
+                // borrowing through self, which causes the compiler to
+                // extend the lifetime of the smaller slice to the lifetime
+                // of self. however, the smaller slice is not dependent on
+                // anything about self except the original slice, so it is
+                // safe to forcibly reduce its lifetime to that of the
+                // original
+                self[i] =
+                    unsafe { mem::transmute::<&mut [u8], &'a mut [u8]>(&mut self[i][..want]) };
+
+                return &mut self[..(i + 1)];
+            }
+
+            want -= buf_len;
+        }
+
+        self
+    }
+}
+
 pub struct Buffer {
     buf: Vec<u8>,
     start: usize,
@@ -193,10 +307,12 @@ impl Buffer {
 #[cfg(test)]
 impl Read for Buffer {
     fn read(&mut self, buf: &mut [u8]) -> Result<usize, io::Error> {
-        let src = self.read_buf();
+        // fully qualified to work around future method warning
+        // https://github.com/rust-lang/rust/issues/48919
+        let src = Buffer::read_buf(self);
         let size = cmp::min(src.len(), buf.len());
 
-        &buf[..size].copy_from_slice(&src[..size]);
+        buf[..size].copy_from_slice(&src[..size]);
 
         self.read_commit(size);
 
@@ -213,7 +329,7 @@ impl Write for Buffer {
         let dest = self.write_buf();
         let size = cmp::min(dest.len(), buf.len());
 
-        &dest[..size].copy_from_slice(&buf[..size]);
+        dest[..size].copy_from_slice(&buf[..size]);
 
         self.write_commit(size);
 
@@ -237,29 +353,45 @@ impl TmpBuffer {
     }
 }
 
-pub struct RingBuffer {
-    buf: Vec<u8>,
+// holds a Vec<u8> but only exposes the portion of it considered to be
+// readable ("filled"). any remaining bytes may be zeroed or uninitialized
+// and are not considered to be readable
+pub struct FilledBuf {
+    data: Vec<u8>,
+    filled: usize,
+}
+
+impl FilledBuf {
+    // panics if filled is larger than data.len()
+    pub fn new(data: Vec<u8>, filled: usize) -> Self {
+        assert!(filled <= data.len());
+
+        Self { data, filled }
+    }
+
+    pub fn filled(&self) -> &[u8] {
+        &self.data[..self.filled]
+    }
+
+    pub fn filled_len(&self) -> usize {
+        self.filled
+    }
+
+    pub fn into_inner(self) -> Vec<u8> {
+        self.data
+    }
+}
+
+pub struct BaseRingBuffer<T> {
+    buf: T,
     start: usize,
     end: usize,
     tmp: Rc<TmpBuffer>,
 }
 
-impl RingBuffer {
-    pub fn new(size: usize, tmp: &Rc<TmpBuffer>) -> RingBuffer {
-        assert!(size <= tmp.len());
-
-        let buf = vec![0; size];
-
-        RingBuffer {
-            buf,
-            start: 0,
-            end: 0,
-            tmp: Rc::clone(tmp),
-        }
-    }
-
+impl<T: AsRef<[u8]> + AsMut<[u8]>> BaseRingBuffer<T> {
     pub fn capacity(&self) -> usize {
-        self.buf.len()
+        self.buf.as_ref().len()
     }
 
     pub fn clear(&mut self) {
@@ -283,72 +415,83 @@ impl RingBuffer {
     }
 
     pub fn read_buf(&self) -> &[u8] {
-        let end = cmp::min(self.end, self.buf.len());
+        let buf = self.buf.as_ref();
+        let end = cmp::min(self.end, buf.len());
 
-        &self.buf[self.start..end]
+        &buf[self.start..end]
     }
 
     pub fn read_buf_mut(&mut self) -> &mut [u8] {
-        let end = cmp::min(self.end, self.buf.len());
+        let buf = self.buf.as_mut();
+        let end = cmp::min(self.end, buf.len());
 
-        &mut self.buf[self.start..end]
+        &mut buf[self.start..end]
     }
 
     pub fn read_commit(&mut self, amount: usize) {
         assert!(self.start + amount <= self.end);
 
+        let buf = self.buf.as_ref();
+
         self.start += amount;
 
         if self.start == self.end {
             self.start = 0;
             self.end = 0;
-        } else if self.start >= self.buf.len() {
-            self.start -= self.buf.len();
-            self.end -= self.buf.len();
+        } else if self.start >= buf.len() {
+            self.start -= buf.len();
+            self.end -= buf.len();
         }
     }
 
     pub fn write_avail(&self) -> usize {
-        self.buf.len() - (self.end - self.start)
+        self.buf.as_ref().len() - (self.end - self.start)
     }
 
     pub fn write_buf(&mut self) -> &mut [u8] {
-        let (start, end) = if self.end < self.buf.len() {
-            (self.end, self.buf.len())
+        let buf = self.buf.as_mut();
+
+        let (start, end) = if self.end < buf.len() {
+            (self.end, buf.len())
         } else {
-            (self.end - self.buf.len(), self.start)
+            (self.end - buf.len(), self.start)
         };
 
-        &mut self.buf[start..end]
+        &mut buf[start..end]
     }
 
     pub fn write_commit(&mut self, amount: usize) {
-        assert!((self.end - self.start) + amount <= self.buf.len());
+        assert!((self.end - self.start) + amount <= self.buf.as_ref().len());
 
         self.end += amount;
     }
 
+    // return true if the readable bytes have not wrapped
+    pub fn is_readable_contiguous(&self) -> bool {
+        self.end <= self.buf.as_ref().len()
+    }
+
     pub fn align(&mut self) -> usize {
         if self.start == 0 {
             return 0;
         }
 
+        let buf = self.buf.as_mut();
         let size = self.end - self.start;
 
-        if self.end <= self.buf.len() {
+        if self.end <= buf.len() {
             // if the buffer hasn't wrapped, simply copy down
-            self.buf.copy_within(self.start.., 0);
+            buf.copy_within(self.start.., 0);
         } else if size <= self.start {
             // if the buffer has wrapped, but the wrapped part can be copied
             //   without overlapping, then copy the wrapped part followed by
             //   initial part
 
-            let left_size = self.end - self.buf.len();
-            let right_size = self.buf.len() - self.start;
+            let left_size = self.end - buf.len();
+            let right_size = buf.len() - self.start;
 
-            self.buf.copy_within(..left_size, right_size);
-            self.buf
-                .copy_within(self.start..(self.start + right_size), 0);
+            buf.copy_within(..left_size, right_size);
+            buf.copy_within(self.start..(self.start + right_size), 0);
         } else {
             // if the buffer has wrapped and the wrapped part can't be copied
             //   without overlapping, then use a temporary buffer to
@@ -357,8 +500,8 @@ impl RingBuffer {
             //   their intended locations. in the worst case, up to 50% of
             //   the buffer may be copied twice
 
-            let left_size = self.end - self.buf.len();
-            let right_size = self.buf.len() - self.start;
+            let left_size = self.end - buf.len();
+            let right_size = buf.len() - self.start;
 
             let (lsize, lsrc, ldest, hsize, hsrc, hdest);
 
@@ -380,9 +523,9 @@ impl RingBuffer {
 
             let mut tmp = self.tmp.0.borrow_mut();
 
-            &mut tmp[..lsize].copy_from_slice(&self.buf[lsrc..(lsrc + lsize)]);
-            self.buf.copy_within(hsrc..(hsrc + hsize), hdest);
-            &mut self.buf[ldest..(ldest + lsize)].copy_from_slice(&mut tmp[..lsize]);
+            tmp[..lsize].copy_from_slice(&buf[lsrc..(lsrc + lsize)]);
+            buf.copy_within(hsrc..(hsrc + hsize), hdest);
+            buf[ldest..(ldest + lsize)].copy_from_slice(&mut tmp[..lsize]);
         }
 
         self.start = 0;
@@ -390,18 +533,24 @@ impl RingBuffer {
 
         size
     }
+
+    pub fn get_tmp(&self) -> &Rc<TmpBuffer> {
+        &self.tmp
+    }
 }
 
 #[cfg(test)]
-impl Read for RingBuffer {
+impl<T: AsRef<[u8]> + AsMut<[u8]>> Read for BaseRingBuffer<T> {
     fn read(&mut self, buf: &mut [u8]) -> Result<usize, io::Error> {
         let mut pos = 0;
 
         while pos < buf.len() && self.read_avail() > 0 {
-            let src = self.read_buf();
+            // fully qualified to work around future method warning
+            // https://github.com/rust-lang/rust/issues/48919
+            let src = Self::read_buf(self);
             let size = cmp::min(src.len(), buf.len() - pos);
 
-            &buf[pos..(pos + size)].copy_from_slice(&src[..size]);
+            buf[pos..(pos + size)].copy_from_slice(&src[..size]);
 
             self.read_commit(size);
 
@@ -412,7 +561,7 @@ impl Read for RingBuffer {
     }
 }
 
-impl Write for RingBuffer {
+impl<T: AsRef<[u8]> + AsMut<[u8]>> Write for BaseRingBuffer<T> {
     fn write(&mut self, buf: &[u8]) -> Result<usize, io::Error> {
         if !buf.is_empty() && self.write_avail() == 0 {
             return Err(io::Error::from(io::ErrorKind::WriteZero));
@@ -424,7 +573,7 @@ impl Write for RingBuffer {
             let dest = self.write_buf();
             let size = cmp::min(dest.len(), buf.len() - pos);
 
-            &dest[..size].copy_from_slice(&buf[pos..(pos + size)]);
+            dest[..size].copy_from_slice(&buf[pos..(pos + size)]);
 
             self.write_commit(size);
 
@@ -439,7 +588,11 @@ impl Write for RingBuffer {
     }
 }
 
-impl RefRead for RingBuffer {
+impl<T: AsRef<[u8]> + AsMut<[u8]>> RefRead for BaseRingBuffer<T> {
+    fn len(&self) -> usize {
+        self.read_avail()
+    }
+
     fn get_ref(&self) -> &[u8] {
         self.read_buf()
     }
@@ -458,17 +611,18 @@ impl RefRead for RingBuffer {
     ) -> &'bufs mut [&'data [u8]] {
         assert!(bufs.len() >= 1);
 
-        let buf_len = self.buf.len();
+        let buf = self.buf.as_ref();
+        let buf_len = buf.len();
 
         if self.end > buf_len && bufs.len() >= 2 {
-            let (part1, part2) = self.buf.split_at(self.start);
+            let (part1, part2) = buf.split_at(self.start);
 
             bufs[0] = part2;
             bufs[1] = &part1[..(self.end - buf_len)];
 
             &mut bufs[..2]
         } else {
-            bufs[0] = &self.buf[self.start..self.end];
+            bufs[0] = &buf[self.start..self.end];
 
             &mut bufs[..1]
         }
@@ -476,27 +630,94 @@ impl RefRead for RingBuffer {
 
     fn get_mut_vectored<'data, 'bufs>(
         &'data mut self,
-        bufs: &'bufs mut [&'data mut [u8]],
+        bufs: &'bufs mut MaybeUninit<[&'data mut [u8]; VECTORED_MAX]>,
     ) -> &'bufs mut [&'data mut [u8]] {
-        assert!(bufs.len() >= 1);
+        let bufs = unsafe { bufs.assume_init_mut() };
 
-        let buf_len = self.buf.len();
+        let buf = self.buf.as_mut();
+        let buf_len = buf.len();
 
         if self.end > buf_len && bufs.len() >= 2 {
-            let (part1, part2) = self.buf.split_at_mut(self.start);
+            let (part1, part2) = buf.split_at_mut(self.start);
 
             bufs[0] = part2;
             bufs[1] = &mut part1[..(self.end - buf_len)];
 
             &mut bufs[..2]
         } else {
-            bufs[0] = &mut self.buf[self.start..self.end];
+            bufs[0] = &mut buf[self.start..self.end];
 
             &mut bufs[..1]
         }
     }
 }
 
+impl BaseRingBuffer<Vec<u8>> {
+    pub fn new(size: usize, tmp: &Rc<TmpBuffer>) -> Self {
+        assert!(size <= tmp.len());
+
+        let buf = vec![0; size];
+
+        BaseRingBuffer {
+            buf,
+            start: 0,
+            end: 0,
+            tmp: Rc::clone(tmp),
+        }
+    }
+
+    // extract inner buffer, aligning it first if necessary, and replace it
+    // with an empty buffer. this should be cheap if the inner buffer is
+    // already aligned. afterwards, the ringbuffer will have a capacity of
+    // zero and will be essentially unusable until set_inner is called with a
+    // non-empty buffer
+    pub fn take_inner(&mut self) -> FilledBuf {
+        self.align();
+
+        let data = mem::take(&mut self.buf);
+        let filled = self.end;
+        self.end = 0;
+
+        FilledBuf::new(data, filled)
+    }
+
+    // replace the inner buffer. this should be cheap if the original inner
+    // buffer is empty, which is the case if take_inner was called earlier.
+    // panics if the new buffer is larger than the tmp buffer
+    pub fn set_inner(&mut self, buf: FilledBuf) {
+        let filled = buf.filled_len();
+        let data = buf.into_inner();
+
+        assert!(data.len() <= self.tmp.len());
+
+        self.buf = data;
+        self.start = 0;
+        self.end = filled;
+    }
+
+    pub fn swap_inner(&mut self, other: &mut Self) {
+        let buf = self.take_inner();
+        self.set_inner(other.take_inner());
+        other.set_inner(buf);
+    }
+}
+
+impl<'a> BaseRingBuffer<&'a mut [u8]> {
+    pub fn new(buf: &'a mut [u8], tmp: &Rc<TmpBuffer>) -> Self {
+        assert!(buf.len() <= tmp.len());
+
+        BaseRingBuffer {
+            buf,
+            start: 0,
+            end: 0,
+            tmp: Rc::clone(tmp),
+        }
+    }
+}
+
+pub type RingBuffer = BaseRingBuffer<Vec<u8>>;
+pub type SliceRingBuffer<'a> = BaseRingBuffer<&'a mut [u8]>;
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -738,4 +959,20 @@ mod tests {
         assert_eq!(r.read_avail(), 6);
         assert_eq!(r.read_buf().len(), 6);
     }
+
+    #[test]
+    fn test_slice_ringbuffer() {
+        let mut buf = [0; 8];
+        let mut backing_buf = [0; 8];
+
+        let tmp = Rc::new(TmpBuffer::new(8));
+
+        let mut r = SliceRingBuffer::new(&mut backing_buf, &tmp);
+        r.write(b"12345678").unwrap();
+        let size = r.read(&mut buf[..4]).unwrap();
+        assert_eq!(&buf[..size], b"1234");
+        r.write(b"90ab").unwrap();
+        let size = r.read(&mut buf).unwrap();
+        assert_eq!(&buf[..size], b"567890ab");
+    }
 }
