@@ -17,7 +17,6 @@
 use crate::arena;
 use crate::event;
 use crate::list;
-use mio;
 use slab::Slab;
 use std::cell::RefCell;
 use std::collections::VecDeque;
@@ -355,6 +354,8 @@ impl<T> LocalSender<T> {
         }
     }
 
+    // NOTE: if the receiver is dropped while there are multiple senders,
+    // only one of the senders will be notified of the disconnect
     pub fn try_clone(
         &self,
         memory: &Rc<arena::RcMemory<event::LocalRegistrationEntry>>,
@@ -369,6 +370,25 @@ impl<T> LocalSender<T> {
             write_registration: write_reg,
         })
     }
+
+    // returns error if a receiver already exists
+    pub fn make_receiver(
+        &self,
+        memory: &Rc<arena::RcMemory<event::LocalRegistrationEntry>>,
+    ) -> Result<LocalReceiver<T>, ()> {
+        if self.channel.read_set_readiness.borrow().is_some() {
+            return Err(());
+        }
+
+        let (read_reg, read_sr) = event::LocalRegistration::new(memory);
+
+        *self.channel.read_set_readiness.borrow_mut() = Some(read_sr);
+
+        Ok(LocalReceiver {
+            channel: self.channel.clone(),
+            read_registration: read_reg,
+        })
+    }
 }
 
 impl<T> Drop for LocalSender<T> {
@@ -406,11 +426,18 @@ impl<T> LocalReceiver<T> {
 
         Ok(value)
     }
+
+    pub fn clear(&self) {
+        // loop over try_recv() in order to notify senders
+        while self.try_recv().is_ok() {}
+    }
 }
 
 impl<T> Drop for LocalReceiver<T> {
     fn drop(&mut self) {
         *self.channel.read_set_readiness.borrow_mut() = None;
+
+        self.channel.notify_one_sender();
     }
 }
 
