@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2020-2021 Fanout, Inc.
+ * Copyright (C) 2020-2022 Fanout, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -25,7 +25,9 @@ pub mod future;
 pub mod http1;
 pub mod list;
 pub mod listener;
+pub mod net;
 pub mod reactor;
+pub mod resolver;
 pub mod server;
 pub mod shuffle;
 pub mod timer;
@@ -41,6 +43,21 @@ use app::Config;
 use log::info;
 use std::error::Error;
 
+// from futures-rs
+#[macro_export]
+macro_rules! pin_mut {
+    ($($x:ident),* $(,)?) => { $(
+        // Move the value to ensure that it is owned
+        let mut $x = $x;
+        // Shadow the original binding so that it can't be directly accessed
+        // ever again.
+        #[allow(unused_mut)]
+        let mut $x = unsafe {
+            Pin::new_unchecked(&mut $x)
+        };
+    )* }
+}
+
 pub fn run(config: &Config) -> Result<(), Box<dyn Error>> {
     info!("starting...");
 
