@@ -18,9 +18,8 @@ use crate::arena;
 use crate::event;
 use crate::event::ReadinessExt;
 use crate::timer::TimerWheel;
-use mio;
 use slab::Slab;
-use std::cell::RefCell;
+use std::cell::{Cell, RefCell};
 use std::cmp;
 use std::io;
 use std::os::unix::io::RawFd;
@@ -29,6 +28,7 @@ use std::task::Waker;
 use std::time::{Duration, Instant};
 
 const TICK_DURATION_MS: u64 = 10;
+const EXPIRE_MAX: usize = 100;
 
 thread_local! {
     static REACTOR: RefCell<Option<Weak<ReactorData>>> = RefCell::new(None);
@@ -58,6 +58,15 @@ impl Registration {
         Reactor { inner: reactor }
     }
 
+    pub fn set_any_as_all(&self, enabled: bool) {
+        let reactor = self.reactor.upgrade().expect("reactor is gone");
+        let registrations = &mut *reactor.registrations.borrow_mut();
+
+        let reg_data = &mut registrations[self.key];
+
+        reg_data.any_as_all = enabled;
+    }
+
     pub fn readiness(&self) -> event::Readiness {
         let reactor = self.reactor.upgrade().expect("reactor is gone");
         let registrations = &*reactor.registrations.borrow();
@@ -101,12 +110,25 @@ impl Registration {
         self.set_readiness(readiness);
     }
 
-    pub fn set_waker(&self, waker: Waker, interest: mio::Interest) {
+    pub fn set_waker(&self, waker: &Waker, interest: mio::Interest) {
         let reactor = self.reactor.upgrade().expect("reactor is gone");
         let registrations = &mut *reactor.registrations.borrow_mut();
 
         let reg_data = &mut registrations[self.key];
 
+        let waker = if let Some((current_waker, _)) = reg_data.waker.take() {
+            if current_waker.will_wake(waker) {
+                // keep the current waker
+                current_waker
+            } else {
+                // switch to the new waker
+                waker.clone()
+            }
+        } else {
+            // we didn't have a waker yet, so we'll use this one
+            waker.clone()
+        };
+
         reg_data.waker = Some((waker, interest));
     }
 
@@ -119,6 +141,20 @@ impl Registration {
         reg_data.waker = None;
     }
 
+    pub fn clear_waker_interest(&self, interest: mio::Interest) {
+        let reactor = self.reactor.upgrade().expect("reactor is gone");
+        let registrations = &mut *reactor.registrations.borrow_mut();
+
+        let reg_data = &mut registrations[self.key];
+
+        if let Some((waker, cur)) = reg_data.waker.take() {
+            match cur.remove(interest) {
+                Some(interest) => reg_data.waker = Some((waker, interest)),
+                None => reg_data.waker = None,
+            }
+        }
+    }
+
     pub fn deregister_io<S: mio::event::Source>(&self, source: &mut S) -> Result<(), io::Error> {
         let reactor = self.reactor.upgrade().expect("reactor is gone");
         let poll = &reactor.poll.borrow();
@@ -126,6 +162,16 @@ impl Registration {
         poll.deregister(source)
     }
 
+    pub fn deregister_custom_local(
+        &self,
+        handle: &event::LocalRegistration,
+    ) -> Result<(), io::Error> {
+        let reactor = self.reactor.upgrade().expect("reactor is gone");
+        let poll = &reactor.poll.borrow();
+
+        poll.deregister_custom_local(handle)
+    }
+
     pub fn pull_from_budget(&self) -> bool {
         let reactor = self.reactor.upgrade().expect("reactor is gone");
 
@@ -160,6 +206,30 @@ impl Registration {
 
         ok
     }
+
+    pub fn reregister_timer(&self, expires: Instant) -> Result<(), io::Error> {
+        let reactor = self.reactor.upgrade().expect("reactor is gone");
+
+        let registrations = &mut *reactor.registrations.borrow_mut();
+        let reg_data = &mut registrations[self.key];
+
+        let timer = &mut *reactor.timer.borrow_mut();
+
+        if let Some(timer_key) = reg_data.timer_key {
+            timer.wheel.remove(timer_key);
+        }
+
+        let expires_ticks = duration_to_ticks_round_up(expires - timer.start);
+
+        let timer_key = match timer.wheel.add(expires_ticks, self.key) {
+            Ok(timer_key) => timer_key,
+            Err(_) => return Err(io::Error::from(io::ErrorKind::Other)),
+        };
+
+        reg_data.timer_key = Some(timer_key);
+
+        Ok(())
+    }
 }
 
 impl Drop for Registration {
@@ -182,6 +252,7 @@ struct RegistrationData {
     readiness: event::Readiness,
     waker: Option<(Waker, mio::Interest)>,
     timer_key: Option<usize>,
+    any_as_all: bool,
 }
 
 struct TimerData {
@@ -250,6 +321,7 @@ impl Reactor {
             readiness: None,
             waker: None,
             timer_key: None,
+            any_as_all: false,
         });
 
         if let Err(e) = self
@@ -284,6 +356,7 @@ impl Reactor {
             readiness: None,
             waker: None,
             timer_key: None,
+            any_as_all: false,
         });
 
         if let Err(e) =
@@ -318,6 +391,7 @@ impl Reactor {
             readiness: None,
             waker: None,
             timer_key: None,
+            any_as_all: false,
         });
 
         if let Err(e) =
@@ -348,6 +422,7 @@ impl Reactor {
             readiness: None,
             waker: None,
             timer_key: None,
+            any_as_all: false,
         });
 
         let timer = &mut *self.inner.timer.borrow_mut();
@@ -458,19 +533,29 @@ impl Reactor {
             let key = key - 1;
 
             if let Some(event_reg) = registrations.get_mut(key) {
-                let changed = {
-                    let readiness = event_reg.readiness;
-                    event_reg.readiness.merge(event.readiness());
-
-                    readiness != event_reg.readiness
+                let event_readiness = if event_reg.any_as_all {
+                    mio::Interest::READABLE | mio::Interest::WRITABLE
+                } else {
+                    event.readiness()
                 };
 
-                if changed {
-                    if let Some((_, interest)) = &event_reg.waker {
-                        let readiness = event.readiness();
+                let (became_readable, became_writable) = {
+                    let prev_readiness = event_reg.readiness;
 
-                        if (readiness.is_readable() && interest.is_readable())
-                            || (readiness.is_writable() && interest.is_writable())
+                    event_reg.readiness.merge(event_readiness);
+
+                    (
+                        !prev_readiness.contains_any(mio::Interest::READABLE)
+                            && event_reg.readiness.contains_any(mio::Interest::READABLE),
+                        !prev_readiness.contains_any(mio::Interest::WRITABLE)
+                            && event_reg.readiness.contains_any(mio::Interest::WRITABLE),
+                    )
+                };
+
+                if became_readable || became_writable {
+                    if let Some((_, interest)) = &event_reg.waker {
+                        if (became_readable && interest.is_readable())
+                            || (became_writable && interest.is_writable())
                         {
                             let (waker, _) = event_reg.waker.take().unwrap();
                             waker.wake();
@@ -482,6 +567,8 @@ impl Reactor {
 
         let timer = &mut *self.inner.timer.borrow_mut();
 
+        let mut expire_count = 0;
+
         while let Some((_, key)) = timer.wheel.take_expired() {
             if let Some(event_reg) = registrations.get_mut(key) {
                 event_reg.readiness = Some(mio::Interest::READABLE);
@@ -491,6 +578,12 @@ impl Reactor {
                     waker.wake();
                 }
             }
+
+            expire_count += 1;
+
+            if expire_count >= EXPIRE_MAX {
+                break;
+            }
         }
     }
 }
@@ -598,7 +691,7 @@ impl CustomEvented {
 
 pub struct TimerEvented {
     registration: Registration,
-    expires: Instant,
+    expires: Cell<Instant>,
 }
 
 impl TimerEvented {
@@ -607,7 +700,7 @@ impl TimerEvented {
 
         Ok(Self {
             registration,
-            expires,
+            expires: Cell::new(expires),
         })
     }
 
@@ -616,7 +709,17 @@ impl TimerEvented {
     }
 
     pub fn expires(&self) -> Instant {
-        self.expires
+        self.expires.get()
+    }
+
+    pub fn set_expires(&self, expires: Instant) -> Result<(), io::Error> {
+        if self.expires.get() == expires {
+            // no change
+            return Ok(());
+        }
+
+        self.expires.set(expires);
+        self.registration.reregister_timer(expires)
     }
 }
 
@@ -671,7 +774,7 @@ mod tests {
 
         evented
             .registration()
-            .set_waker(waker.clone().into_std(), mio::Interest::READABLE);
+            .set_waker(&waker.clone().into_std(), mio::Interest::READABLE);
 
         let thread = thread::spawn(move || {
             std::net::TcpStream::connect(addr).unwrap();
@@ -702,7 +805,7 @@ mod tests {
 
         evented
             .registration()
-            .set_waker(waker.clone().into_std(), mio::Interest::READABLE);
+            .set_waker(&waker.clone().into_std(), mio::Interest::READABLE);
 
         let thread = thread::spawn(move || {
             std::net::TcpStream::connect(addr).unwrap();
@@ -729,7 +832,7 @@ mod tests {
 
         evented
             .registration()
-            .set_waker(waker.clone().into_std(), mio::Interest::READABLE);
+            .set_waker(&waker.clone().into_std(), mio::Interest::READABLE);
 
         let thread = thread::spawn(move || {
             sr.set_readiness(mio::Interest::READABLE).unwrap();
@@ -756,7 +859,7 @@ mod tests {
 
         evented
             .registration()
-            .set_waker(waker.clone().into_std(), mio::Interest::READABLE);
+            .set_waker(&waker.clone().into_std(), mio::Interest::READABLE);
 
         assert_eq!(waker.was_waked(), false);
         assert_eq!(reactor.now(), now);
@@ -815,7 +918,7 @@ mod tests {
 
         evented
             .registration()
-            .set_waker(waker.clone().into_std(), mio::Interest::READABLE);
+            .set_waker(&waker.clone().into_std(), mio::Interest::READABLE);
 
         assert_eq!(evented.registration().pull_from_budget(), true);
         assert_eq!(waker.was_waked(), false);
@@ -830,7 +933,7 @@ mod tests {
         reactor.set_budget(Some(1));
         evented
             .registration()
-            .set_waker(waker.clone().into_std(), mio::Interest::READABLE);
+            .set_waker(&waker.clone().into_std(), mio::Interest::READABLE);
 
         assert_eq!(evented.registration().pull_from_budget(), true);
         assert_eq!(waker.was_waked(), false);
