@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2020-2021 Fanout, Inc.
+ * Copyright (C) 2020-2022 Fanout, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,6 +15,7 @@
  */
 
 use crate::arena;
+use crate::buffer::trim_for_display;
 use crate::channel;
 use crate::event;
 use crate::executor::Executor;
@@ -24,6 +25,7 @@ use crate::future::{
     REGISTRATIONS_PER_ZMQSOCKET,
 };
 use crate::list;
+use crate::pin_mut;
 use crate::reactor::Reactor;
 use crate::tnetstring;
 use crate::zhttppacket::{Id, Response, ResponseScratch};
@@ -50,37 +52,6 @@ const LOG_METADATA_MAX: usize = 1_000;
 const LOG_CONTENT_MAX: usize = 1_000;
 const EXECUTOR_TASKS_MAX: usize = 1;
 
-fn trim(s: &str, max: usize) -> String {
-    // NOTE: O(n)
-    let char_len = s.chars().count();
-
-    if char_len > max && max >= 7 {
-        let dist = max / 2;
-        let mut left_end = 0;
-        let mut right_start = 0;
-
-        // NOTE: O(n)
-        for (i, (pos, _)) in s.char_indices().enumerate() {
-            // dist guaranteed to be < char_len
-            if i == dist {
-                left_end = pos;
-            }
-
-            // (char_len - dist + 3) guaranteed to be < char_len
-            if i == char_len - dist + 3 {
-                right_start = pos;
-            }
-        }
-
-        let left = &s[..left_end];
-        let right = &s[right_start..];
-
-        format!("{}...{}", left, right)
-    } else {
-        s.to_owned()
-    }
-}
-
 struct Packet<'a> {
     map_frame: tnetstring::Frame<'a>,
     content_field: Option<&'a str>,
@@ -178,7 +149,7 @@ impl fmt::Display for Packet<'_> {
         // formatted output is guaranteed to be utf8
         let meta = String::from_utf8(meta).unwrap();
 
-        let meta = trim(&meta, LOG_METADATA_MAX);
+        let meta = trim_for_display(&meta, LOG_METADATA_MAX);
 
         if self.content_field.is_some() {
             let mut content = Vec::new();
@@ -192,7 +163,7 @@ impl fmt::Display for Packet<'_> {
                 // formatted output is guaranteed to be utf8
                 let content = String::from_utf8(content).unwrap();
 
-                let content = trim(&content, LOG_CONTENT_MAX);
+                let content = trim_for_display(&content, LOG_CONTENT_MAX);
 
                 return write!(f, "{} {} {}", meta, clen, content);
             }
@@ -294,7 +265,7 @@ struct ReqPipeEnd {
 struct StreamPipeEnd {
     sender: channel::Sender<arena::Arc<zmq::Message>>,
     receiver_any: channel::Receiver<zmq::Message>,
-    receiver_addr: channel::Receiver<(ArrayVec<[u8; 64]>, zmq::Message)>,
+    receiver_addr: channel::Receiver<(ArrayVec<u8, 64>, zmq::Message)>,
 }
 
 struct AsyncReqPipeEnd {
@@ -305,28 +276,28 @@ struct AsyncReqPipeEnd {
 struct AsyncStreamPipeEnd {
     sender: AsyncSender<arena::Arc<zmq::Message>>,
     receiver_any: AsyncReceiver<zmq::Message>,
-    receiver_addr: AsyncReceiver<(ArrayVec<[u8; 64]>, zmq::Message)>,
+    receiver_addr: AsyncReceiver<(ArrayVec<u8, 64>, zmq::Message)>,
 }
 
 enum ControlRequest {
     Stop,
     SetClientReq(Vec<SpecInfo>),
     SetClientStream(Vec<SpecInfo>, Vec<SpecInfo>, Vec<SpecInfo>),
-    AddClientReqHandle(ReqPipeEnd, ArrayString<[u8; 8]>),
-    AddClientStreamHandle(StreamPipeEnd, ArrayString<[u8; 8]>),
+    AddClientReqHandle(ReqPipeEnd, ArrayString<8>),
+    AddClientStreamHandle(StreamPipeEnd, ArrayString<8>),
 }
 
 type ControlResponse = Result<(), String>;
 
 struct ReqPipe {
     pe: AsyncReqPipeEnd,
-    filter: ArrayString<[u8; 8]>,
+    filter: ArrayString<8>,
     valid: Cell<bool>,
 }
 
 struct StreamPipe {
     pe: AsyncStreamPipeEnd,
-    filter: ArrayString<[u8; 8]>,
+    filter: ArrayString<8>,
     valid: Cell<bool>,
 }
 
@@ -339,13 +310,12 @@ impl<T> Future for RecvWrapperFuture<'_, T> {
     type Output = (usize, Result<T, mpsc::RecvError>);
 
     fn poll(mut self: Pin<&mut Self>, cx: &mut Context) -> Poll<Self::Output> {
-        let nkey = self.nkey;
-        let fut = unsafe { self.as_mut().map_unchecked_mut(|s| &mut s.fut) };
+        let r = &mut *self;
 
-        match fut.poll(cx) {
+        match Pin::new(&mut r.fut).poll(cx) {
             Poll::Ready(result) => match result {
-                Ok(value) => Poll::Ready((nkey, Ok(value))),
-                Err(mpsc::RecvError) => Poll::Ready((nkey, Err(mpsc::RecvError))),
+                Ok(value) => Poll::Ready((r.nkey, Ok(value))),
+                Err(mpsc::RecvError) => Poll::Ready((r.nkey, Err(mpsc::RecvError))),
             },
             Poll::Pending => Poll::Pending,
         }
@@ -398,7 +368,7 @@ impl ReqHandles {
         self.nodes.len()
     }
 
-    fn add(&mut self, pe: AsyncReqPipeEnd, filter: ArrayString<[u8; 8]>) {
+    fn add(&mut self, pe: AsyncReqPipeEnd, filter: ArrayString<8>) {
         assert!(self.nodes.len() < self.nodes.capacity());
 
         let key = self.nodes.insert(list::Node::new(ReqPipe {
@@ -513,7 +483,7 @@ struct StreamHandles {
     nodes: Slab<list::Node<StreamPipe>>,
     list: list::List,
     recv_any_scratch: RefCell<RecvScratch<zmq::Message>>,
-    recv_addr_scratch: RefCell<RecvScratch<(ArrayVec<[u8; 64]>, zmq::Message)>>,
+    recv_addr_scratch: RefCell<RecvScratch<(ArrayVec<u8, 64>, zmq::Message)>>,
     need_cleanup: Cell<bool>,
 }
 
@@ -532,7 +502,7 @@ impl StreamHandles {
         self.nodes.len()
     }
 
-    fn add(&mut self, pe: AsyncStreamPipeEnd, filter: ArrayString<[u8; 8]>) {
+    fn add(&mut self, pe: AsyncStreamPipeEnd, filter: ArrayString<8>) {
         assert!(self.nodes.len() < self.nodes.capacity());
 
         let key = self.nodes.insert(list::Node::new(StreamPipe {
@@ -582,7 +552,7 @@ impl StreamHandles {
         }
     }
 
-    async fn recv_addr(&self) -> (ArrayVec<[u8; 64]>, zmq::Message) {
+    async fn recv_addr(&self) -> (ArrayVec<u8, 64>, zmq::Message) {
         let mut scratch = self.recv_addr_scratch.borrow_mut();
 
         let (mut tasks, slice_scratch) = scratch.get();
@@ -710,28 +680,31 @@ impl SocketManager {
 
         let instance_id = String::from(instance_id);
 
-        let thread = thread::spawn(move || {
-            debug!("manager thread start");
+        let thread = thread::Builder::new()
+            .name("zhttpsocket".to_string())
+            .spawn(move || {
+                debug!("manager thread start");
 
-            // 2 control channels, 3 channels per handle, 4 zmq sockets
-            let channels = 2 + (HANDLES_MAX * 3);
-            let zmqsockets = 4;
+                // 2 control channels, 3 channels per handle, 4 zmq sockets
+                let channels = 2 + (HANDLES_MAX * 3);
+                let zmqsockets = 4;
 
-            let registrations_max =
-                (channels * REGISTRATIONS_PER_CHANNEL) + (zmqsockets * REGISTRATIONS_PER_ZMQSOCKET);
+                let registrations_max = (channels * REGISTRATIONS_PER_CHANNEL)
+                    + (zmqsockets * REGISTRATIONS_PER_ZMQSOCKET);
 
-            let reactor = Reactor::new(registrations_max);
+                let reactor = Reactor::new(registrations_max);
 
-            let executor = Executor::new(EXECUTOR_TASKS_MAX);
+                let executor = Executor::new(EXECUTOR_TASKS_MAX);
 
-            executor
-                .spawn(Self::run(ctx, s1, r2, instance_id, retained_max, hwm))
-                .unwrap();
+                executor
+                    .spawn(Self::run(ctx, s1, r2, instance_id, retained_max, hwm))
+                    .unwrap();
 
-            executor.run(|timeout| reactor.poll(timeout)).unwrap();
+                executor.run(|timeout| reactor.poll(timeout)).unwrap();
 
-            debug!("manager thread end");
-        });
+                debug!("manager thread end");
+            })
+            .unwrap();
 
         Self {
             handle_bound,
@@ -923,18 +896,26 @@ impl SocketManager {
                 None
             };
 
-            let result = select_9(
-                control_receiver.recv(),
-                select_option(req_handles_recv),
-                select_option(req_send.as_mut()),
-                client_req.sock.recv_routed(),
-                select_option(stream_handles_recv_any),
-                select_option(stream_out_send.as_mut()),
-                select_option(stream_handles_recv_addr),
-                select_option(stream_out_stream_send.as_mut()),
-                client_stream.in_.recv(),
-            )
-            .await;
+            let result = {
+                pin_mut!(
+                    req_handles_recv,
+                    stream_handles_recv_any,
+                    stream_handles_recv_addr
+                );
+
+                select_9(
+                    control_receiver.recv(),
+                    select_option(req_handles_recv.as_pin_mut()),
+                    select_option(req_send.as_mut()),
+                    client_req.sock.recv_routed(),
+                    select_option(stream_handles_recv_any.as_pin_mut()),
+                    select_option(stream_out_send.as_mut()),
+                    select_option(stream_handles_recv_addr.as_pin_mut()),
+                    select_option(stream_out_stream_send.as_mut()),
+                    client_stream.in_.recv(),
+                )
+                .await
+            };
 
             match result {
                 // control_receiver.recv
@@ -1281,7 +1262,7 @@ impl AsyncClientReqHandle {
 
 pub struct ClientStreamHandle {
     sender_any: channel::Sender<zmq::Message>,
-    sender_addr: channel::Sender<(ArrayVec<[u8; 64]>, zmq::Message)>,
+    sender_addr: channel::Sender<(ArrayVec<u8, 64>, zmq::Message)>,
     receiver: channel::Receiver<arena::Arc<zmq::Message>>,
 }
 
@@ -1336,7 +1317,7 @@ impl ClientStreamHandle {
 
 pub struct AsyncClientStreamHandle {
     sender_any: AsyncSender<zmq::Message>,
-    sender_addr: AsyncSender<(ArrayVec<[u8; 64]>, zmq::Message)>,
+    sender_addr: AsyncSender<(ArrayVec<u8, 64>, zmq::Message)>,
     receiver: AsyncReceiver<arena::Arc<zmq::Message>>,
 }
 
@@ -1365,7 +1346,7 @@ impl AsyncClientStreamHandle {
 
     pub async fn send_to_addr(
         &self,
-        addr: ArrayVec<[u8; 64]>,
+        addr: ArrayVec<u8, 64>,
         msg: zmq::Message,
     ) -> Result<(), io::Error> {
         match self.sender_addr.send((addr, msg)).await {
